﻿using Bonsai.IO;
using OpenCV.Net;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reactive.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;

namespace Shane.Writers
{
    public class MultiChannelFileWriter : FileSink<Mat, MultiChannelWriter>
    {
        protected override MultiChannelWriter CreateWriter(string fileName, Mat input)
        {
            return new MultiChannelWriter(fileName, input, Overwrite);
        }

        protected override void Write(MultiChannelWriter writer, Mat input)
        {
            writer.Write(input);
        }
    }

    public class MultiChannelWriter : IDisposable
    {
        BinaryWriter[] writers;

        public MultiChannelWriter(string fileName, Mat input, bool overwrite)
        {
            writers = new BinaryWriter[input.Rows];
            for (int i = 0; i < writers.Length; i++)
            {
                var path = PathHelper.AppendSuffix(fileName, i.ToString());
                var stream = new FileStream(path, overwrite ? FileMode.Create : FileMode.CreateNew);
                writers[i] = new BinaryWriter(stream);
            }
        }

        public void Write(Mat input)
        {
            for (int i = 0; i < writers.Length; i++)
            {
                var channel = input.GetRow(i);
                var data = ToArray(channel);
                writers[i].Write(data);
            }
        }

        static byte[] ToArray(Mat input)
        {
            var step = input.ElementSize * input.Cols;
            var data = new byte[step * input.Rows];
            var dataHandle = GCHandle.Alloc(data, GCHandleType.Pinned);
            try
            {
                var dataHeader = new Mat(input.Rows, input.Cols, input.Depth, input.Channels, dataHandle.AddrOfPinnedObject(), step);
                CV.Copy(input, dataHeader);
            }
            finally { dataHandle.Free(); }
            return data;
        }

        public void Dispose()
        {
            for (int i = 0; i < writers.Length; i++)
            {
                writers[i].Close();
            }
        }
    }
}
